@@ -1,15 +1,14 @@
-# $Id: TLConfFile.pm 29725 2013-04-07 18:58:34Z karl $
+# $Id: TLConfFile.pm 59226 2021-05-16 18:22:05Z karl $
 # TeXLive::TLConfFile.pm - reading and writing conf files
-# Copyright 2010, 2011, 2012 Norbert Preining
+# Copyright 2010-2021 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLConfFile;
 
 use TeXLive::TLUtils;
-use File::Temp qw/tempfile/;
 
-my $svnrev = '$Revision: 29725 $';
+my $svnrev = '$Revision: 59226 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -194,7 +193,7 @@ sub value
       } else {
         # select element based on first-win or last-win type
         my $ll = $key_lines[($t eq 'first-win' ? 0 : $#key_lines)];
-        print "lastwin = $ll\n";
+        #print "lastwin = $ll\n";
         if ($self{'keyvalue'}{$key}{$ll}{'value'} ne $value) {
           $self{'keyvalue'}{$key}{$ll}{'value'} = $value;
           # as long as the key/value pair is not new,
@@ -215,6 +214,7 @@ sub value
       $self{'changed'} = 1;
     }
   }
+  # $self->dump_myself();
   if (defined($self{'keyvalue'}{$key})) {
     my @key_lines = sort by_lnr CORE::keys %{$self{'keyvalue'}{$key}};
     if ($t eq 'first-win') {
@@ -314,31 +314,35 @@ sub save
   #
   # first we write the config file as close as possible to orginal layout,
   # and after that we add new key/value pairs
+  my $current_key_value_is_changed = 0;
   for my $i (0..$config{'lines'}) {
-    my $is_changed = 0;
     if ($config{$i}{'type'} eq 'comment') {
       print $fhout "$config{$i}{'value'}";
       print $fhout ($config{$i}{'multiline'} ? "\\\n" : "\n");
     } elsif ($config{$i}{'type'} eq 'empty') {
       print $fhout ($config{$i}{'multiline'} ? "\\\n" : "\n");
     } elsif ($config{$i}{'type'} eq 'data') {
+      $current_key_value_is_changed = 0;
       # we have to check whether the original data has been changed!!
       if ($self{'keyvalue'}{$config{$i}{'key'}}{$i}{'status'} eq 'changed') {
-        $is_changed = 1;
+        $current_key_value_is_changed = 1;
         print $fhout "$config{$i}{'key'} $config{'sep'} $self{'keyvalue'}{$config{$i}{'key'}}{$i}{'value'}";
         if (defined($config{$i}{'postcomment'})) {
           print $fhout $config{$i}{'postcomment'};
         }
-        print $fhout ($config{$i}{'multiline'} ? "\\\n" : "\n");
+        # if a value is changed, we do not print out multiline stuff
+        # as keys are not split
+        print $fhout "\n";
       } elsif ($self{'keyvalue'}{$config{$i}{'key'}}{$i}{'status'} eq 'deleted') {
-        $is_changed = 1;
+        $current_key_value_is_changed = 1;
       } else {
-        print $fhout "$config{$i}{'original'}";
-        print $fhout ($config{$i}{'multiline'} ? "\\\n" : "\n");
+        $current_key_value_is_changed = 0;
+        # the original already contains the final \, so only print new line
+        print $fhout "$config{$i}{'original'}\n";
       }
     } elsif ($config{$i}{'type'} eq 'continuation') {
-      if ($is_changed) {
-        # ignore continuation lines
+      if ($current_key_value_is_changed) {
+        # ignore continuation lines if values are changed
       } else {
         print $fhout "$config{$i}{'value'}";
         print $fhout ($config{$i}{'multiline'} ? "\\\n" : "\n");
@@ -476,9 +480,28 @@ sub parse_config_file {
   }
   # save the number of lines in the config hash
   $config{'lines'} = $lines;
+  #print "====DEBUG dumping config ====\n";
+  #dump_config_data(\%config);
+  #print "====DEBUG writing config ====\n";
+  #write_config_file(\%config);
+  #print "=============================\n";
   return %config;
 }
 
+sub dump_myself {
+  my $self = shift;
+  print "======== DUMPING SELF =============\n";
+  dump_config_data($self{'confdata'});
+  print "DUMPING KEY VALUES\n";
+  for my $k (CORE::keys %{$self{'keyvalue'}}) {
+    print "key = $k\n";
+    for my $l (sort CORE::keys %{$self{'keyvalue'}{$k}}) {
+      print "  line =$l= value =", $self{'keyvalue'}{$k}{$l}{'value'}, "= status =", $self{'keyvalue'}{$k}{$l}{'status'}, "=\n";
+    }
+  }
+  print "=========== END DUMP ==============\n";
+}
+
 sub dump_config_data {
   my $foo = shift;
   my %config = %{$foo};
@@ -489,14 +512,16 @@ sub dump_config_data {
   for my $i (0..$config{'lines'}) {
     print "line ", $i+1, ": $config{$i}{'type'}";
     if ($config{$i}{'type'} eq 'comment') {
-      print "\nCOMMNENT = $config{$i}{'value'}\n";
+      print "\nCOMMENT = $config{$i}{'value'}\n";
     } elsif ($config{$i}{'type'} eq 'data') {
       print "\nKEY = $config{$i}{'key'}\nVALUE = $config{$i}{'value'}\n";
+      print "MULTLINE = ", ($config{$i}{'multiline'} ? "1" : "0"), "\n";
     } elsif ($config{$i}{'type'} eq 'empty') {
       print "\n";
       # do nothing
     } elsif ($config{$i}{'type'} eq 'continuation') {
       print "\nVALUE = $config{$i}{'value'}\n";
+      print "MULTLINE = ", ($config{$i}{'multiline'} ? "1" : "0"), "\n";
     } else {
       print "-- UNKNOWN TYPE\n";
     }
@@ -534,13 +559,14 @@ __END__
 
 =head1 NAME
 
-C<TeXLive::TLConfFile> -- TeX Live Config File Access Module
+C<TeXLive::TLConfFile> -- TeX Live generic configuration files
 
 =head1 SYNOPSIS
 
   use TeXLive::TLConfFile;
 
-  $conffile = TeXLive::TLConfFile->new($file_name, $comment_char, $separator, $type);
+  my $conffile = TeXLive::TLConfFile->new($file_name, $comment_char,
+                                          $separator, $type);
   $conffile->file;
   $conffile->cc;
   $conffile->sep;
@@ -681,7 +707,7 @@ After that you can query keys:
 =head1 AUTHORS AND COPYRIGHT
 
 This script and its documentation were written for the TeX Live
-distribution (L<http://tug.org/texlive>) and both are licensed under the
+distribution (L<https://tug.org/texlive>) and both are licensed under the
 GNU General Public License Version 2 or later.
 
 =cut
