@@ -1,6 +1,6 @@
 #!/usr/bin/env wish
 
-# Copyright 2018-2020 Siep Kroonenberg
+# Copyright 2018-2021 Siep Kroonenberg
 
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
@@ -37,15 +37,21 @@ catch {rename send {}}
 
 # This file should be in $::instroot/tlpkg/installer.
 # On non-windows platforms, install-tl functions as a wrapper
-# for this file if it encounters a parameter '-gui tcl'.
+# for this file if it encounters a parameter '-gui' (but not -gui text).
 # This allows automatic inclusion of a '--' parameter to separate
 # tcl parameters from script parameters.
 
 set ::instroot [file normalize [info script]]
 set ::instroot [file dirname [file dirname [file dirname $::instroot]]]
 
-# declarations, initializations and procs shared with tlshell.tcl
+# declarations, initializations and procs shared with tlshell.tcl.
+# tltcl may want to know whether or not it was invoked by tlshell:
+set ::invoker [file tail [info script]]
+if [string match -nocase ".tcl" [string range $::invoker end-3 end]] {
+  set ::invoker [string range $::invoker 0 end-4]
+}
 source [file join $::instroot "tlpkg" "tltcl" "tltcl.tcl"]
+unset ::invoker
 
 ### initialize some globals ###
 
@@ -64,7 +70,7 @@ if {$::tcl_platform(platform) eq "windows"} {
 set ::advanced 0
 set ::alltrees 0
 
-# interactively select repository
+# interactively select repository; default false
 set ::select_repo 0
 
 proc kill_perl {} {
@@ -144,52 +150,7 @@ proc dblog {s} {
   close $db
 }
 
-proc maybe_print_welcome {} {
-  # if the last non-empty line was "All done", then installation is completed.
-  # otherwise, it was help output or an interrupted installation.
-  # we allow for spurious empty lines at the end of the backend output.
-
-  set all_done 0
-  for {set i [.log.tx count -lines 1.0 end]} {$i > 0} {incr i -1} {
-    set l  [.log.tx get ${i}.0 ${i}.end]
-    if {$l ne ""} {
-      if {[string range $l 0 11] eq "Installed on"} {
-        set all_done 1
-      }
-      break
-    }
-  }
-  if {!$all_done} return
-  # need TEXDIR and this_platform in case of profile install
-  if {! [info exists ::vars(this_platform)] || \
-          ! [info exists ::vars(TEXDIR)]} {
-    if [regexp {^Installed on platform (.*) at (.*)$} $l m p r] {
-      set ::vars(this_platform) $p
-      set ::vars(TEXDIR) $r
-    } else {
-      set ::vars(this_platform) "PLATFORM"
-      set ::vars(TEXDIR) "ROOT"
-    }
-  }
-
-  .log.tx configure -state normal
-  .log.tx tag configure center -justify center
-  .log.tx delete ${i}.0 end
-  .log.tx insert end "\n\n"
-  .log.tx insert end [__ "Welcome to TeX Live!"] center
-  .log.tx insert end "\n\n"
-  # tags appear to interfere with --/::msgcat::mc !?!
-  set s  [__ "See %s/index.html for links to documentation.\nThe TeX Live web site (https://tug.org/texlive/) contains any updates and corrections. TeX Live is a joint project of the TeX user groups around the world; please consider supporting it by joining the group best for you. The list of groups is available on the web at https://tug.org/usergroups.html." $::vars(TEXDIR)]
-  .log.tx insert end $s center
-  if {$::tcl_platform(platform) ne "windows"} {
-    .log.tx insert end "\n\n"
-    set s [__ "Add %s/texmf-dist/doc/man to MANPATH.\nAdd %s/texmf-dist/doc/info to INFOPATH.\nMost importantly, add %s/bin/%s\nto your PATH for current and future sessions."  $::vars(TEXDIR) $::vars(TEXDIR) $::vars(TEXDIR) $::vars(this_platform)]
-    .log.tx insert end $s center
-  }
-  .log.tx insert end "\n"
-  .log.tx yview moveto 1
-  if {$::tcl_platform(os) ne "Darwin"} {.log.tx configure -state disabled}
-}
+# welcome message now provided by install-tl
 
 # regular read_line
 proc read_line {} {
@@ -250,7 +211,6 @@ proc read_line_cb {} {
     # This closes stdin of the child
     .close state !disabled
     if [winfo exists .abort] {.abort state disabled}
-    maybe_print_welcome
   } elseif {$len >= 0} {
     # regular output
     .log.tx configure -state normal
@@ -301,7 +261,7 @@ proc replace_repo_parameter {m} {
   while {$i > 0} {
     incr i -1
     set p [lindex $::argv $i]
-    if {$p in [list "-location" "-url" "-repository" "-repos" "-repo"]} {
+    if {$p in {"-location" "-url" "-repository" "-repos" "-repo"}} {
       set j [expr {$i+1}]
       if {$j < [llength $::argv]} {
         set ::argv [lreplace $::argv $i $j]
@@ -374,9 +334,9 @@ proc pre_splash {} {
   }
 
   # wallpaper for remaining widgets
-  pack [ttk::frame .bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
 
-  # buttons: abort button, mirrors dropdown menu
+  # frame for buttons (abort button, mirrors dropdown menu)
   pack [ttk::frame .splfb] -in .bg -side bottom -fill x
 }
 
@@ -388,7 +348,6 @@ proc select_mirror {} {
   ppack [ttk::button .splfb.slmir_a -text [__ "Abort"] -command maybe_abort] \
       -side right
 
-  wm attributes . -topmost
   update
   wm state . normal
   raise .
@@ -396,11 +355,12 @@ proc select_mirror {} {
 } ; # select_mirror
 
 proc make_splash {} {
+  wm withdraw .
 
+  # we want this if select_mirror has run:
   foreach c [winfo children .splfb] {
     catch {destroy $c}
   }
-  update
 
   ppack [ttk::button .spl_a -text [__ "Abort"] -command maybe_abort] \
       -side right -in .splfb
@@ -408,7 +368,7 @@ proc make_splash {} {
 
   # some text
   ppack [ttk::label .text -text [__ "TeX Live Installer"] \
-             -font bigfont] -in .bg
+             -font hfont] -in .bg
   if {! $::select_repo} {
     ppack [ttk::label .loading -text [__ "Trying to load %s.
 
@@ -416,7 +376,6 @@ If this takes too long, press Abort or choose another repository." \
                                           $::prelocation]] -in .bg
   }
 
-  wm attributes . -topmost
   update
   wm state . normal
   raise .
@@ -430,7 +389,7 @@ proc show_log {{do_abort 0}} {
   }
 
   # wallpaper
-  pack [ttk::frame .bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
 
   # buttons at bottom
   pack [ttk::frame .bottom] -in .bg -side bottom -fill x
@@ -448,7 +407,7 @@ proc show_log {{do_abort 0}} {
   pack [ttk::frame .log] -in .bg -fill both -expand 1
   pack [ttk::scrollbar .log.scroll -command ".log.tx yview"] \
       -side right -fill y
-  ppack [text .log.tx -height 10 -wrap word -font TkDefaultFont \
+  ppack [text .log.tx -height 20 -wrap word -font TkDefaultFont \
       -yscrollcommand ".log.scroll set"] \
       -expand 1 -fill both
   .log.tx configure -state normal
@@ -461,7 +420,6 @@ proc show_log {{do_abort 0}} {
 
   wm resizable . 1 1
   wm overrideredirect . 0
-  wm attributes . -topmost
   update
   wm state . normal
   raise .
@@ -482,10 +440,6 @@ proc log_exit {{mess ""}} {
   }
 }; # log_exit
 
-#############################################################
-
-##########################################################
-
 ##### installation root #####
 
 proc update_full_path {} {
@@ -517,14 +471,14 @@ proc edit_name {} {
   if $::plain_unix {wm attributes .tled -type dialog}
 
   # wallpaper
-  pack [ttk::frame .tled.bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .tled.bg -padding 3pt] -fill both -expand 1
 
   # widgets
   ttk::label .tled.l -text [__ "Change name (slashes not allowed)"]
-  pack .tled.l -in .tled.bg -padx 5 -pady 5
+  pack .tled.l -in .tled.bg -padx 5pt -pady 5pt
   ttk::entry .tled.e -width 20
   .tled.e state !disabled
-  pack .tled.e -in .tled.bg -pady 5
+  pack .tled.e -in .tled.bg -pady 5pt
   .tled.e insert 0 [.tltd.name_l cget -text]
 
   # now frame with ok and cancel buttons
@@ -538,9 +492,9 @@ proc edit_name {} {
       end_dlg "" .tled
     }
   }
-  ppack .tled.ok_b -in .tled.buttons -side right -padx 5 -pady 5
+  ppack .tled.ok_b -in .tled.buttons -side right -padx 5pt -pady 5pt
   ttk::button .tled.cancel_b -text [__ "Cancel"] -command {end_dlg "" .tled}
-  ppack .tled.cancel_b -in .tled.buttons -side right -padx 5 -pady 5
+  ppack .tled.cancel_b -in .tled.buttons -side right -padx 5pt -pady 5pt
   bind .tled <Escape> {.tled.cancel_b invoke}
 
   wm protocol .tled WM_DELETE_WINDOW \
@@ -597,7 +551,11 @@ proc commit_root {} {
   }
   commit_canonical_local
 
-  if {$::vars(instopt_portable)} reset_personal_dirs
+  if {$::vars(instopt_portable)} {
+    set ::vars(TEXMFHOME) $::vars(TEXMFLOCAL)
+    set ::vars(TEXMFVAR) $::vars(TEXMFSYSVAR)
+    set ::vars(TEXMFCONFIG) $::vars(TEXMFSYSCONFIG)
+  }
   destroy .tltd
 }
 
@@ -611,14 +569,14 @@ proc texdir_setup {} {
   wm title .tltd [__ "Installation root"]
 
   # wallpaper
-  pack [ttk::frame .tltd.bg -padding 3] -expand 1 -fill both
+  pack [ttk::frame .tltd.bg -padding 3pt] -expand 1 -fill both
 
   # full path
   pack [ttk::label .tltd.path_l -font lfont -anchor center] \
-      -in .tltd.bg -pady 10 -fill x -expand 1
+      -in .tltd.bg -pady 10pt -fill x -expand 1
 
   # installation root components, gridded
-  pack [ttk::frame .tltd.fr1 -borderwidth 2 -relief groove] \
+  pack [ttk::frame .tltd.fr1 -borderwidth 2pt -relief groove] \
       -in .tltd.bg -fill x -expand 1
   grid columnconfigure .tltd.fr1 0 -weight 1
   grid columnconfigure .tltd.fr1 2 -weight 1
@@ -654,7 +612,7 @@ proc texdir_setup {} {
   }
 
   # ok/cancel buttons
-  pack [ttk::frame .tltd.frbt] -in .tltd.bg -pady [list 10 0] -fill x
+  pack [ttk::frame .tltd.frbt] -in .tltd.bg -pady {10pt 0pt} -fill x
   ttk::button .tltd.ok_b -text [__ "Ok"] -command commit_root
   ppack .tltd.ok_b -in .tltd.frbt -side right
   ttk::button .tltd.cancel_b -text [__ "Cancel"] \
@@ -718,7 +676,7 @@ proc edit_dir {d} {
   if $::plain_unix {wm attributes .td -type dialog}
 
   # wallpaper
-  pack [ttk::frame .td.bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .td.bg -padding 3pt] -fill both -expand 1
 
   if {$d eq "TEXMFHOME"} {
     # explain tilde
@@ -923,7 +881,7 @@ proc select_binaries {} {
   wm title .tlbin [__ "Binaries"]
 
   # wallpaper
-  pack [ttk::frame .tlbin.bg -padding 3] -expand 1 -fill both
+  pack [ttk::frame .tlbin.bg -padding 3pt] -expand 1 -fill both
 
   # ok, cancel buttons
   pack [ttk::frame .tlbin.buts] -in .tlbin.bg -side bottom -fill x
@@ -934,13 +892,6 @@ proc select_binaries {} {
   ppack .tlbin.cancel -in .tlbin.buts -side right
   bind .tlbin <Escape> {.tlbin.cancel invoke}
 
-  #set max_width 0
-  #foreach b [array names ::bin_descs] {
-  #  set bl [font measure TkTextFont [__ $::bin_descs($b)]]
-  #  if {$bl > $max_width} {set max_width $bl}
-  #}
-  #incr max_width 10
-
   # treeview for binaries, with checkbox column and vertical scrollbar
   pack [ttk::frame .tlbin.binsf] -in .tlbin.bg -expand 1 -fill both
 
@@ -955,6 +906,7 @@ proc select_binaries {} {
     .tlbin.lst insert {}  end -id $b -values \
         [list [mark_sym $::vars($bb)] [__ $::bin_descs($b)]]
   }
+  set_tree_col_width .tlbin.lst "desc"
   pgrid .tlbin.lst -in .tlbin.binsf -row 0 -column 0 -sticky news
   pgrid .tlbin.binsc -in .tlbin.binsf -row 0 -column 1 -sticky ns
   grid columnconfigure .tlbin.binsf 0 -weight 1
@@ -979,7 +931,7 @@ proc select_scheme {} {
   wm title .tlschm [__ "Schemes"]
 
   # wallpaper
-  pack [ttk::frame .tlschm.bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .tlschm.bg -padding 3pt] -fill both -expand 1
 
   # buttons at bottom
   pack [ttk::frame .tlschm.buts] -in .tlschm.bg -side bottom -fill x
@@ -1004,20 +956,14 @@ proc select_scheme {} {
   ppack .tlschm.cancel -in .tlschm.buts -side right
   bind .tlschm <Escape> {.tlschm.cancel invoke}
 
-  # schemes list
-  #set max_width 0
-  #foreach s $::schemes_order {
-  #  set sl [font measure TkTextFont [__ $::scheme_descs($s)]]
-  #  if {$sl > $max_width} {set max_width $sl}
-  #}
-  #incr max_width 10
+  # schemes list. use treeview rather than listbox for uniform formatting
   ttk::treeview .tlschm.lst -columns {desc} -show {} -selectmode browse \
       -height [llength $::schemes_order]
-  .tlschm.lst column "desc" -stretch 1; # -minwidth $max_width
   ppack .tlschm.lst -in .tlschm.bg -fill both -expand 1
   foreach s $::schemes_order {
     .tlschm.lst insert {} end -id $s -values [list [__ $::scheme_descs($s)]]
   }
+  set_tree_col_width .tlschm.lst "desc"
   # we already made sure that $::vars(selected_scheme) has a valid value
   .tlschm.lst selection set [list $::vars(selected_scheme)]
 
@@ -1073,7 +1019,7 @@ proc select_collections {} {
   wm title .tlcoll [__ "Collections"]
 
   # wallpaper
-  pack [ttk::frame .tlcoll.bg -padding 3] -fill both -expand 1
+  pack [ttk::frame .tlcoll.bg -padding 3pt] -fill both -expand 1
 
   # frame at bottom with ok and cancel buttons
   pack [ttk::frame .tlcoll.butf] -in .tlcoll.bg -side bottom -fill x
@@ -1093,14 +1039,14 @@ proc select_collections {} {
     set wgb .tlcoll.b$t
     ttk::frame $wgb
     ttk::label ${wgb}sel -text [__ "Select"]
-    ttk::button ${wgb}all -text [__ "All"] -padding 1 -command \
+    ttk::button ${wgb}all -text [__ "All"] -padding 1pt -command \
       "foreach c \[.tlcoll.$t children {}\] \{
         .tlcoll.$t set \$c mk \[mark_sym 1\]\}"
-    ttk::button ${wgb}none -text [__ "None"] -padding 1 -command \
+    ttk::button ${wgb}none -text [__ "None"] -padding 1pt -command \
       "foreach c \[.tlcoll.$t children {}\] \{
         .tlcoll.$t set \$c mk \[mark_sym 0\]\}"
     pack ${wgb}sel ${wgb}all ${wgb}none -in $wgb \
-        -side left -padx 3 -pady 3
+        -side left -padx 3pt -pady 3pt
 
     # trees with collections and markers, lang and other separately
     set wgt ".tlcoll.$t"
@@ -1142,6 +1088,8 @@ proc select_collections {} {
     $wgt insert {} end -id $c -values \
         [list [mark_sym $::vars($c)] [__ $::coll_descs($c)]]
   }
+  set_tree_col_width .tlcoll.lang "desc"
+  set_tree_col_width .tlcoll.other "desc"
 
   wm protocol .tlcoll WM_DELETE_WINDOW \
       {cancel_or_destroy .tlcoll.cancel .tlcoll}
@@ -1257,7 +1205,7 @@ if {$::tcl_platform(platform) ne "windows"} {
     create_dlg .edsyms .
     wm title .edsyms [__ "Symlinks"]
 
-    pack [ttk::frame .edsyms.bg -padding 3] -expand 1 -fill both
+    pack [ttk::frame .edsyms.bg -padding 3pt] -expand 1 -fill both
     set rw -1
 
     pack [ttk::frame .edsyms.fr0] -in .edsyms.bg -expand 1 -fill both
@@ -1315,40 +1263,127 @@ if {$::tcl_platform(platform) ne "windows"} {
 
 #############################################################
 
+proc set_language {l} {
+  set ::lang $l
+  load_translations
+  run_menu
+}
+
+proc set_fontscale {s} {
+  set ::tkfontscale $s
+  redo_fonts
+  run_menu
+}
+
+proc zoom {n} {
+  if {$n <= 0} {set n 1}
+  set_fontscale [expr {$n*$::tkfontscale}]
+}
+
+# menus: disable tearoff feature
+option add *Menu.tearOff 0
+
+#############################################################
+
 # the main menu interface will at certain events send the current values of
 # the ::vars array to install-tl[-tcl], which will send back an updated version
 # of this array.
 # We still use blocking i/o: frontend and backend wait for each other.
 
-# idea: follow submenu organization of text installer
-# for 3-way options, create an extra level of children
-# instead of wizard install, supppress some options
+## default_bg color, only used for menus under ::plain_unix
+if [catch {ttk::style lookup TFrame -background} ::default_bg] {
+  set ::default_bg white
+}
+
+proc abort_menu {} {
+  set ::out_log {}
+  set ::menu_ans "no_inst"
+  # i.e. anything but advanced, alltrees or startinst
+}
 
 proc run_menu {} {
-  if [info exists ::env(dbgui)] {
-    #puts "\ndbgui: run_menu: advanced is now $::advanced"
-    #puts "dbgui: run_menu: alltrees is now $::alltrees"
-  }
+  #if [info exists ::env(dbgui)] {
+  #  puts "\ndbgui: run_menu: advanced is now $::advanced"
+  #  puts "dbgui: run_menu: alltrees is now $::alltrees"
+  #}
   wm withdraw .
   foreach c [winfo children .] {
     catch {destroy $c}
   }
 
-  # wallpaper
-  pack [ttk::frame .bg -padding 3] -fill both -expand 1
+  if $::plain_unix {
+    # plain_unix: avoid a possible RenderBadPicture error on quitting
+    # when there is a menu.
+    # 'send' bypasses the bug by changing the shutdown sequence.
+    # 'tk appname <something>' restores 'send'.
+    bind . <Destroy> {
+      catch {tk appname appname}
+    }
+  }
+
+  # name in titlebar; should be redefined after any language switch
+  wm title . [__ "TeX Live Installer"]
+
+  # menu, for language selection and font scaling
+  menu .mn
+  . configure -menu .mn
+  if $::plain_unix {
+    .mn configure -borderwidth 1pt
+    .mn configure -background $::default_bg
+  }
+  .mn add command -command abort_menu -label [__ "Abort"]
+
+
+  if {[llength $::langs] > 1} {
+    menu .mn.lang
+    .mn add cascade -label [__ "GUI language"] -menu .mn.lang
+    foreach l [lsort $::langs] {
+      if {$l eq $::lang} {
+        set mlabel "$l *"
+      } else {
+        set mlabel $l
+      }
+      .mn.lang add command -label $mlabel -command "set_language $l"
+    }
+  }
+
+  menu .mn.fscale
+  .mn add cascade -label [__ "Font scaling"] -menu .mn.fscale
+  .mn.fscale add command -label \
+      "[__ "Current:"] [format { %.2f} $::tkfontscale]"
+  foreach s {0.6 0.8 1 1.2 1.6 2 2.5 3 3.8 5 6 7.5 9} {
+    .mn.fscale add command -label $s -command "set_fontscale $s"
+  }
+
+  # browser-style keyboard shortcuts for scaling
+  bind . <Control-KeyRelease-minus> {zoom 0.8}
+  bind . <Control-KeyRelease-equal> {zoom 1.25}
+  bind . <Control-Shift-KeyRelease-equal> {zoom 1.25}
+  bind . <Control-KeyRelease-plus> {zoom 1.25}
+  bind . <Control-KeyRelease-0> {set_fontscale 1}
+  if {$::tcl_platform(os) eq "Darwin"} {
+    bind . <Command-KeyRelease-minus> {zoom 0.8}
+    bind . <Command-KeyRelease-equal> {zoom 1.25}
+    bind . <Command-Shift-KeyRelease-equal> {zoom 1.25}
+    bind . <Command-KeyRelease-plus> {zoom 1.25}
+    bind . <Command-KeyRelease-0> {set_fontscale 1}
+  }
+
+  # wallpaper, for a uniform background
+  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
 
   # title
   ttk::label .title -text [__ "TeX Live %s Installer" $::release_year] \
       -font titlefont
-  pack .title -pady {10 1} -in .bg
+  pack .title -pady {10pt 1pt} -in .bg
   pack [ttk::label .svn -text "r. $::svn"] -in .bg
 
   pack [ttk::separator .seph0 -orient horizontal] \
-      -in .bg -pady 3 -fill x
+      -in .bg -pady 3pt -fill x
 
   # frame at bottom with install/quit buttons
   pack [ttk::frame .final] \
-      -in .bg -side bottom -pady [list 5 2] -fill x
+      -in .bg -side bottom -pady {5pt 2pt} -fill x
   ppack [ttk::button .install -text [__ "Install"] -command {
     set ::menu_ans "startinst"}] -in .final -side right
   ppack [ttk::button .quit -text [__ "Quit"] -command {
@@ -1362,7 +1397,7 @@ proc run_menu {} {
     }] -in .final -side left
   }
   pack [ttk::separator .seph1 -orient horizontal] \
-      -in .bg -side bottom -pady 3 -fill x
+      -in .bg -side bottom -pady 3pt -fill x
 
   # directories, selections
   # advanced and basic have different frame setups
@@ -1551,7 +1586,7 @@ proc run_menu {} {
   if $::advanced {
 
     pack [ttk::separator .sepv -orient vertical] \
-        -in .bg -side left -padx 3 -fill y
+        -in .bg -side left -padx 3pt -fill y
     pack [ttk::frame .options] -in .bg -side right -fill both -expand 1
 
     set curf .options
@@ -1566,7 +1601,7 @@ proc run_menu {} {
   }
 
   # instopt_letter
-  set ::lpapers [list "A4" "letter"]
+  set ::lpapers {"A4" "Letter"}
   incr rw
   pgrid [ttk::label .paperl -text [__ "Default paper size"]] \
       -in $curf -row $rw -column 0 -sticky w
@@ -1663,14 +1698,9 @@ proc run_menu {} {
     pgrid [ttk::label .texwl -text [__ "Install TeXworks front end"]] \
         -in $curf -row $rw -column 0 -columnspan 2 -sticky w
     ttk::checkbutton .texwb -variable ::vars(collection-texworks)
+    .texwb configure -command \
+        {set ::vars(selected_scheme) "scheme-custom"; update_vars; show_stats}
     pgrid .texwb -in $curf -row $rw -column 2 -sticky e
-    bind .texwb <ButtonRelease> {+
-      set ::vars(selected_scheme) "scheme-custom"; update_vars; show_stats}
-    bind .texwb <Return> {+
-      set ::vars(selected_scheme) "scheme-custom"; update_vars; show_stats}
-    bind .texwb <space> {+
-      set ::vars(selected_scheme) "scheme-custom"; update_vars; show_stats}
-
   } else {
     if $::advanced {
       # instopt_adjustpath, unix edition: symlinks
@@ -1706,11 +1736,11 @@ proc run_menu {} {
   if $::advanced {port_dis_or_activate 0}
   show_stats
   wm overrideredirect . 0
-  wm attributes . -topmost
   wm resizable . 0 0
   update
   wm state . normal
   raise .
+  if {$::tcl_platform(platform) eq "windows"} {wm deiconify .}
   if {[is_nonempty $::vars(TEXDIR)] && ! $::td_warned} {
     td_warn $::vars(TEXDIR)
   }
@@ -1881,15 +1911,15 @@ proc run_installer {} {
   set ::out_log {}
   show_log 1; # 1: with abort button
   .close state disabled
-  if $::did_gui {
-    chan puts $::inst "startinst"
-    write_vars
-  }
+  chan puts $::inst "startinst"
+  write_vars
   # the backend was already running and needs no further encouragement
 
   # switch to non-blocking i/o
   chan configure $::inst -buffering line -blocking 0
   chan event $::inst readable read_line_cb
+  raise .
+  if {$::tcl_platform(platform) eq "windows"} {wm deiconify .}
 }; # run_installer
 
 proc whataboutclose {} {
@@ -1908,40 +1938,28 @@ proc whataboutclose {} {
 
 proc main_prog {} {
 
-  wm title . [__ "TeX Live Installer"]
   wm protocol . WM_DELETE_WINDOW whataboutclose
 
-  if {[file exists $::dblfile]} {file delete $::dblfile}
-
   # handle some command-line arguments.
   # the argument list should already be normalized: '--' => '-', "=" => ' '
-  set i 0
-  set do_splash 1
   set ::prelocation "..."
   set ::mir_selected 1 ; # i.e. default or set by parameter
-  set i [llength $::argv]
+  set l [llength $::argv]
+  set i $l
   while {$i > 0} {
+    set iplus $i
     incr i -1
     set p [lindex $::argv $i]
-    if {$p eq "-profile"} {
-      # check for profile argument: no splash screen if present
-      set do_splash 0
-    } elseif {$p in [list "-location" "-url" "-repository" "-repos" "-repo"]} {
+    if {$p in {"-location" "-url" "-repository" "-repos" "-repo"}} {
       # check for repository argument: bail out if obviously invalid
-      set j [expr {$i+1}]
-      if {$j<[llength $::argv]} {
-        set p [lindex $::argv $j]
-        unset j
+      if {$iplus<$l} {
+        set p [lindex $::argv $iplus]
         if {$p ne "ctan" && ! [possible_repository $p]} {
-          tk_messageBox -message [__ "%s not a local or remote repository" $p] \
-              -title [__ "Error"] -type ok -icon error
-          exit
+          err_exit [__ "%s not a local or remote repository" $p]
         }
         set ::prelocation $p
       } else {
-        tk_messageBox -message [__ "%s requires an argument" $p] \
-            -title [__ "Error"] -type ok -icon error
-        exit
+        err_exit [__ "%s requires an argument" $p]
       }
     } elseif {$p eq "-select-repository"} {
       # in this case, we start with selecting a repository
@@ -1954,23 +1972,16 @@ proc main_prog {} {
   }
   unset i
 
-  if {$do_splash || ! [info exists ::mir_selected]} pre_splash
-
+  pre_splash
   if {! [info exists ::mir_selected]} {
     select_mirror
+    # waits for ::mir_selected
   }
-
-  if $do_splash make_splash
-  unset do_splash
-
-  if {! [info exist ::mir_selected]} {
-    vwait ::mir_selected
-  }
+  make_splash
 
   # start install-tl-[tcl] via a pipe.
   set cmd [list "|${::perlbin}" "${::instroot}/install-tl" \
-               "-from_ext_gui" {*}$::argv 2>@1]
-  show_time "opening pipe"
+               "-from_ext_gui" {*}$::argv "2>@1"]
   if [catch {open $cmd r+} ::inst] {
     err_exit "Error starting Perl backend"
   }
@@ -1979,15 +1990,14 @@ proc main_prog {} {
   set ::perlpid [pid $::inst]
 
   # for windows < 10: make sure the main window is still on top
-  wm attributes . -topmost
+  raise .
 
   chan configure $::inst -buffering line -blocking 1
 
   # possible input from perl until the menu starts:
   # - question about prior canceled installation
   # - location (actual repository)
-  # - menu data, help, version, print-platform
-  set ::did_gui 0
+  # - menu data
   set answer ""
   unset -nocomplain ::loaded
   while 1 { ; # initial perl output
@@ -2019,7 +2029,6 @@ If this takes too long, press Abort or choose another repository." \
   # waiting till the repository has been loaded
   vwait ::loaded
   unset ::loaded
-  #puts stderr "done loading"
   # resume reading from back end in blocking mode
   while 1 {
     set ll [read_line]
@@ -2047,7 +2056,6 @@ If this takes too long, press Abort or choose another repository." \
           set answer [run_menu]
         }
       }
-      set ::did_gui 1
       break
     } elseif {$l eq "startinst"} {
       # use an existing profile:
@@ -2059,6 +2067,20 @@ If this takes too long, press Abort or choose another repository." \
     }
   }
   if {$answer eq "startinst"} {
+  # disable browser-style keyboard shortcuts for scaling
+    bind . <Control-KeyRelease-minus> {}
+    bind . <Control-KeyRelease-equal> {}
+    bind . <Control-Shift-KeyRelease-equal> {}
+    bind . <Control-KeyRelease-plus> {}
+    bind . <Control-KeyRelease-0> {}
+    if {$::tcl_platform(os) eq "Darwin"} {
+      bind . <Command-KeyRelease-minus> {}
+      bind . <Command-KeyRelease-equal> {}
+      bind . <Command-Shift-KeyRelease-equal> {}
+      bind . <Command-KeyRelease-plus> {}
+      bind . <Command-KeyRelease-0> {}
+    }
+
     run_installer
     # invokes show_log which first destroys previous children
   } else {
